package com.citizen.app.layout.javasample;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.citizen.sdk.CitizenPrinterInfo;
import com.citizen.sdk.ESCPOSConst;
import com.citizen.sdk.ESCPOSPrinter;
import com.citizen.sdk.LayoutPrintEngine;

import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;

/**
 * <p>English:
 * <p>This application is a minimal example of a merge print program that uses the layout files.
 * <p>Required libraries
 * <ul>
 * <li>CSJPOS_CLFX_PrintEngine.jar (Layout SDK)
 * <li> CSJPOSLib_Android.jar (POS Print SDK)
 * </ul>
 * <p>Note
 * <ul>
 * <li> Available layout files are xml format files output using the export function of Layout Editor.
 * <li> Available printers depend on CSJPOSLib_Android.jar. Please use the latest version of CSJPOSLib_Android.jar.
 * <li> Printing, etc. of this application is processed by synchronous call. Each process blocks and waits for the thread to complete. Keep this in mind when porting.
 * </ul>
 * <br><br>
 * <p> Japanese:
 * <p>本アプリケーションは、レイアウト ファイルを利用した差込印刷プログラムの最小限の例です。
 * <p>必要ライブラリ
 * <ul>
 * <li> CSJPOS_CLFX_PrintEngine.jar(Layout SDK)
 * <li> CSJPOSLib_Android.jar(POS Print SDK)
 * </ul>
 * <p>注意
 * <ul>
 * <li> 利用可能なレイアウト ファイルは Layout Editor のエクスポート機能を利用して出力した xml 形式のファイルです。
 * <li> 利用可能なプリンターは CSJPOSLib_Android.jar に依存します。CSJPOSLib_Android.jar は最新のものをご利用ください。
 * <li> 本アプリケーションの印刷等は、同期型呼び出しで処理します。各処理は処理が完了するまでスレッドをブロックして待機します。移植の際は、この点に留意してください。
 * </ul>
 *
 * <p>(C) 2014 CITIZEN SYSTEMS JAPAN CO., LTD. All Rights Reserved.
 */
public class MainActivity extends AppCompatActivity {

	private int getSelectedConnectType() {
		int connectType = ESCPOSConst.CMP_PORT_WiFi;
		Spinner spinnerInterface = findViewById(R.id.spinner_Interface);
		if (spinnerInterface != null) {
			connectType = spinnerInterface.getSelectedItemPosition();
		}
		return connectType;
	}

	private String getAddress() {
		String address = "";
		EditText editTextAddress = findViewById(R.id.editText_Address);
		if (editTextAddress != null) {
			address = editTextAddress.getText().toString();
		}
		return address;
	}

	private void setAddress(String address) {
		EditText editTextAddress = findViewById(R.id.editText_Address);
		if (editTextAddress != null) {
			editTextAddress.setText(address);
		}
	}

	private int getSearchTime(int connectType) {
		int searchTime = 0;
		switch (connectType) {
			case ESCPOSConst.CMP_PORT_WiFi:
					searchTime = 3;
					break;
			case ESCPOSConst.CMP_PORT_Bluetooth:
			case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
					searchTime = 10;
					break;
			case ESCPOSConst.CMP_PORT_USB:
					searchTime = 0;
					break;
		}
		return searchTime;
	}

	private void setAddressHint(int connectType) {
		EditText editTextAddress = findViewById(R.id.editText_Address);
		if (editTextAddress != null) {
			String hint = "";
			switch (connectType) {
				case ESCPOSConst.CMP_PORT_WiFi:
					hint = getString(R.string.hint_address_wifi);
					break;
				case ESCPOSConst.CMP_PORT_Bluetooth:
					hint = getString(R.string.hint_address_bluetooth_secure);
					break;
				case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
					hint = getString(R.string.hint_address_bluetooth_insecure);
					break;
				case ESCPOSConst.CMP_PORT_USB:
					hint = getString(R.string.hint_usb_device);
					break;
			}
			editTextAddress.setHint(hint);
		}
	}


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Software keyboard : Hidden
		getWindow().setSoftInputMode(android.view.WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

		//
		getPermissions(permissions);

		//
		((EditText) findViewById(R.id.editText_Header)).setText(DEFAULT_HEADER);
		((EditText) findViewById(R.id.editText_Item01)).setText(DEFAULT_ITEM01);
		((EditText) findViewById(R.id.editText_Item02)).setText(DEFAULT_ITEM02);
		((EditText) findViewById(R.id.editText_Item03)).setText(DEFAULT_ITEM03);
		((EditText) findViewById(R.id.editText_Item04)).setText(DEFAULT_ITEM04);
		((EditText) findViewById(R.id.editText_Item05)).setText(DEFAULT_ITEM05);
		((EditText) findViewById(R.id.editText_Footer)).setText(DEFAULT_FOOTER);

		// Print Button
		Button buttonPrint = findViewById(R.id.buttonPrint);
		buttonPrint.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(final View v) {
				onPrint();
			}
		});

		// Select IF
		final Spinner spinnerInterface = findViewById(R.id.spinner_Interface);
		if (spinnerInterface != null) {
			spinnerInterface.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
				@Override
				public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
					setAddress(loadAddress(position));
					setAddressHint(position);
				}
				@Override
				public void onNothingSelected(AdapterView<?> parent) {
					Log.e("OnNothingSelected", "NothingSelected");
				}
			});
			//
			spinnerInterface.setSelection(0);
		}

		// Search
		final Button buttonSearch = findViewById(R.id.button_Search);
		if (buttonSearch != null) {
			buttonSearch.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					onSearch(0);
				}
			});
		}
	}


	/**
	 * Search Printer Button
	 * @param searchMode 0 : searchCitizenPrinter(), != 0 : searchESCPOSPrinter()
	 */
	private void onSearch(int searchMode) {

		// Get selected interface
		int connectType = getSelectedConnectType();
		if (connectType == ESCPOSConst.CMP_PORT_USB) {
			// Usb interface is Auto Detect only
			toast("USB interface is Automatic Detection only", Toast.LENGTH_LONG);
			return;
		}

		// Create an instance
		ESCPOSPrinter printer = new ESCPOSPrinter();

		// Set context
		printer.setContext(MainActivity.this);

		// Get timeout
		int timeout = getSearchTime(connectType);

		int[] errCode = new int[1];
		String[] items = null;
		String[] itemsDetail = null;
		// Search
		if (searchMode == 0) {
			// Search Citizen Printer
			CitizenPrinterInfo[] list1 = printer.searchCitizenPrinter(connectType, timeout, errCode);
			if (list1.length > 0) {
				items = new String[list1.length];
				itemsDetail = new String[list1.length];
				for (int i = 0; i < list1.length; i++) {
					if (connectType == ESCPOSConst.CMP_PORT_WiFi) {
						items[i] = list1[i].ipAddress;
						itemsDetail[i] = list1[i].ipAddress + " [" + list1[i].macAddress + "]";
					} else {
						items[i] = list1[i].bdAddress;
						itemsDetail[i] = list1[i].bdAddress + " [" + list1[i].deviceName + "]";
					}
				}
			}
		} else {
			// Search Printer
			String[] list2 = printer.searchESCPOSPrinter(connectType, timeout, errCode);
			if (list2.length > 0) {
				items = list2;
				itemsDetail = list2;
			}
		}

		if (items != null) {
			int checkedItem = -1;
			for (int i = 0; i < items.length; i++) {
				if (items[i].equals(getAddress())) {
					checkedItem = i;
					break;
				}
			}
			// Search address preview and select
			final String[] finalItems = items;
			new AlertDialog.Builder(MainActivity.this)
					.setTitle("Select the address")
					.setSingleChoiceItems(itemsDetail, checkedItem, new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							setAddress(String.format("%s", finalItems[which]));
							dialog.dismiss();
						}
					})
					.show();
		} else {
			// 0 search results
			toast("0 Search Results", Toast.LENGTH_LONG);
		}
	}

	/**
	 * Print Button
	 */
	private void onPrint() {

		// Get the port type.
		int connectType = getSelectedConnectType();

		// Get the address.
		String address = getAddress();

		// Get the layout file.
		Spinner spinnerLayout = findViewById(R.id.spinner_Layout);
		String layoutName = (String) spinnerLayout.getSelectedItem();
		int layoutId = getResources().getIdentifier(layoutName, "raw", getPackageName());

		// Get the setting of upside down.
		CheckBox checkUpsideDown = findViewById(R.id.checkBox_UpsideDown);
		boolean isUpsideDown = checkUpsideDown.isChecked();

		// Get the setting of Logo.
		CheckBox checkReplaceLogo = findViewById(R.id.checkBox_ReplaceLogo);
		boolean isReplaceLogo = checkReplaceLogo.isChecked();

		// Gets the print data.
		String header = ((EditText) findViewById(R.id.editText_Header)).getText().toString();
		String footer = ((EditText) findViewById(R.id.editText_Footer)).getText().toString();
		ArrayList<String> items = new ArrayList<>();
		items.add(((EditText) findViewById(R.id.editText_Item01)).getText().toString());
		items.add(((EditText) findViewById(R.id.editText_Item02)).getText().toString());
		items.add(((EditText) findViewById(R.id.editText_Item03)).getText().toString());
		items.add(((EditText) findViewById(R.id.editText_Item04)).getText().toString());
		items.add(((EditText) findViewById(R.id.editText_Item05)).getText().toString());

		doPrint(connectType, address, layoutId, isUpsideDown, isReplaceLogo, header, footer, items.toArray(new String[0]));

	}


	/**
	 * Make the printing by current UI settings.
	 *
	 * @param connectType   : Port type of the printer. ESCPOSConst.CMP_PORT_WiFi, CMP_PORT_Bluetooth, CMP_PORT_Bluetooth_Insecure, CMP_PORT_USB
	 * @param address       : Address of the printer port.
	 * @param layoutId      : Resource ID of the layout file.
	 * @param isUpsideDown  : Upside down printing.
	 * @param isReplaceLogo : Replace Logo.
	 * @param header        : Header text.
	 * @param footer        : Footer text.
	 * @param items         : Items name.
	 */
	private void doPrint(int connectType, String address, int layoutId, boolean isUpsideDown, boolean isReplaceLogo, String header, String footer, String[] items) {

		InputStream inputStream = getResources().openRawResource(layoutId);

		//
		// LayoutPrintEngine Class.
		//
		LayoutPrintEngine lpe = new LayoutPrintEngine();
		int result = lpe.open(inputStream);
		if (result == 0) {
			lpe.beginPrint();

			//
			// Header frame
			//
			int frameIndex = lpe.initFrame("Frame1");
			lpe.setPartsData(frameIndex, "Text1", header);
			if (isReplaceLogo) {
				android.graphics.BitmapFactory.Options opt = new android.graphics.BitmapFactory.Options();
				opt.inScaled = false;
				android.graphics.Bitmap bmp = android.graphics.BitmapFactory.decodeResource(getResources(), R.raw.newlogo, opt);
				java.io.ByteArrayOutputStream outputStream = new java.io.ByteArrayOutputStream();
				bmp.compress(android.graphics.Bitmap.CompressFormat.PNG, 100, outputStream);
				byte[] bytes = outputStream.toByteArray();
				lpe.setPartsData(frameIndex, "Image1", bytes);
			}
			lpe.addFrame(frameIndex);

			//
			// Items frame
			//
			int digits = 1;
			for (String item : items) {
				frameIndex = lpe.initFrame("Frame2");
				// Set a item name.
				lpe.setPartsData(frameIndex, "Text1", item);
				// Create and set a dummy price. ( ex. 9.99, 99.99, 999.99,,, )
				double price = Math.pow(10.00, digits++) - 0.01;
				String sPrice = new DecimalFormat("#,##0.##").format(price);
				lpe.setPartsData(frameIndex, "Text2", sPrice);
				lpe.addFrame(frameIndex);
			}

			//
			// Footer frame
			//
			frameIndex = lpe.initFrame("Frame3");
			lpe.setPartsData(frameIndex, "Text1", footer);
			lpe.setPartsData(frameIndex, "Barcode1", footer);
			lpe.addFrame(frameIndex);

			//
			// Coupon frame, Please do not execute addFrame method, using initFrame method only.
			//
			lpe.initFrame("Frame4");

			//
			// Output to the printer
			//
			doPrint(connectType, address, lpe, isUpsideDown);

			lpe.endPrint();
			lpe.close();

		} else {
			// Error
			toast("LayoutPrintEngine open() error result : " + result, Toast.LENGTH_LONG);
		}

		try {
			inputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Connects to the printer and outputs the print data.
	 *
	 * @param connectType  : Port type of the printer. </br>ESCPOSConst.CMP_PORT_WiFi, CMP_PORT_Bluetooth, CMP_PORT_Bluetooth_Insecure, CMP_PORT_USB
	 * @param address      : Address of the printer port.
	 * @param lpe          : LayoutPrintEngine object that holds the print layout
	 * @param isUpsideDown : Upside down.
	 * @return result.
	 */
	private int doPrint(int connectType, String address, LayoutPrintEngine lpe, boolean isUpsideDown) {

		int result;

		//
		// ESCPOSPrinter Class.
		//
		ESCPOSPrinter printer = new ESCPOSPrinter();

		// Required to use the USB interface on the printer.
		printer.setContext(getApplicationContext());

		// Character set
//		printer.setEncoding("ISO-8859-1");	// Latin-1
		printer.setEncoding("Shift_JIS");	// Japanese
//		printer.setEncoding("UTF-8");		// Requires a printer that supports UTF-8.

		//
		// Connects to the printer.
		//
		result = printer.connect(connectType, address);
		if (result == ESCPOSConst.CMP_SUCCESS) {

			saveAddress(connectType, address);

			//
			// Outputs the print data.
			//
			result = lpe.doPrint(printer, isUpsideDown);

			//
			// ESCPOSPrinter error handling
			//
			if (result == ESCPOSConst.CMP_E_OFFLINE) {
				printer.clearOutput();
			}

			//
			// Disconnects the printer.
			//
			printer.disconnect();

			if (result == 0) {
				toast("LayoutPrintEngine doPrint() success ", Toast.LENGTH_LONG);
			} else {
				// Error
				toast("LayoutPrintEngine doPrint() error result : " + result, Toast.LENGTH_LONG);
			}
		} else {
			// Error
			toast("ESCPOSPrinter connect() error result : " + result, Toast.LENGTH_LONG);
		}
		return result;
	}


	//
	// Save & Load Address
	//
	private void saveAddress(int connectType, String address) {
		switch (connectType) {
			case ESCPOSConst.CMP_PORT_WiFi:
				savePreferences(PRE_KEY_IP_ADDRESS, address);
				break;
			case ESCPOSConst.CMP_PORT_Bluetooth:
			case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
				savePreferences(PRE_KEY_BLUETOOTH_ADDRESS, address);
				break;
			default:
				break;
		}
	}

	private String loadAddress(int connectType) {
		String address = "";
		switch (connectType) {
			case ESCPOSConst.CMP_PORT_WiFi:
				address = loadPreferences(PRE_KEY_IP_ADDRESS, DEFAULT_IP_ADDRESS);
				break;
			case ESCPOSConst.CMP_PORT_Bluetooth:
			case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
				address = loadPreferences(PRE_KEY_BLUETOOTH_ADDRESS, DEFAULT_BLUETOOTH_ADDRESS);
				break;
			default:
				break;
		}
		return address;
	}

	private void savePreferences(String sKeyName, String sValue) {
		SharedPreferences sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE);
		SharedPreferences.Editor e = sp.edit();
		e.putString(sKeyName, sValue);
		e.apply();
	}

	private String loadPreferences(String sKeyName, String sDefaultValue) {
		SharedPreferences sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE);
		String value = sp.getString(sKeyName, sDefaultValue);
		if (value == null) {
			value = "";
		}
		return value;
	}

	private void toast(String message, int duration) {
		Toast toast = Toast.makeText(getApplicationContext(), message, duration);
		toast.setGravity(Gravity.CENTER, 0, 0);
		toast.show();
	}

	/**
	 * Get the permissions.
	 *
	 * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
	 */
	private void getPermissions(@NonNull final String[] permissions) {

		if (!checkPermissions(permissions)) {
			requestPermissions(permissions);
		} else {
			Log.d("getPermissions", "PERMISSION_GRANTED");
		}
	}

	/**
	 * Checks whether this app has all permissions.
	 *
	 * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
	 * @return true:All permissions granted, false:Permission denied
	 */
	private boolean checkPermissions(@NonNull final String[] permissions) {

		for (String permission : permissions) {
			int permissionResult = ContextCompat.checkSelfPermission(this, permission);
			if (permissionResult != android.content.pm.PackageManager.PERMISSION_GRANTED) {
				return false;                // non-permission
			}
		}
		return true;
	}

	/**
	 * Requests permissions to be granted to this app.
	 *
	 * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
	 */
	private void requestPermissions(@NonNull final String[] permissions) {
		ActivityCompat.requestPermissions(this, permissions, PERMISSION_REQUEST_CODE);
	}



	/**
	 * Permissions
	 * @see <a href="https://developer.android.com/guide/topics/security/permissions#perm-groups">Dangerous permissions and permission groups.</a>
	 */
	private final String[] permissions = {
			android.Manifest.permission.ACCESS_FINE_LOCATION,
//			android.Manifest.permission.WRITE_EXTERNAL_STORAGE,
	};
	/** requestCode for {@link ActivityCompat.OnRequestPermissionsResultCallback} @Override#onRequestPermissionsResult */
	private static final int PERMISSION_REQUEST_CODE = 1234;

	private final String PREFERENCES_NAME = "CitizenLayoutSDKSample";
	private final String PRE_KEY_IP_ADDRESS = "IP_Address";
	private final String DEFAULT_IP_ADDRESS = "192.168.0.100";
	private final String PRE_KEY_BLUETOOTH_ADDRESS = "Bluetooth_Address";
	private final String DEFAULT_BLUETOOTH_ADDRESS = "00:01:90:E8:03:33";

	private final String DEFAULT_HEADER = "CITIZEN";
	private final String DEFAULT_ITEM01 = "Sample ABC";
	private final String DEFAULT_ITEM02 = "Sample 123";
	private final String DEFAULT_ITEM03 = "Sample 456";
	private final String DEFAULT_ITEM04 = "Sample 789";
	private final String DEFAULT_ITEM05 = "Coupon";
	private final String DEFAULT_FOOTER = "https://www.citizen-systems.co.jp";
}
