package com.citizen.app.layout.kotlinsample

import android.app.Activity
import android.os.Bundle
import android.support.v4.app.ActivityCompat
import android.support.v4.content.ContextCompat
import android.support.v7.app.AlertDialog
import android.support.v7.app.AppCompatActivity
import android.util.Log
import android.view.Gravity
import android.view.View
import android.widget.*
import com.citizen.sdk.ESCPOSConst
import com.citizen.sdk.ESCPOSPrinter
import com.citizen.sdk.LayoutPrintEngine
import java.io.IOException
import java.text.DecimalFormat
import java.util.*
import kotlin.math.pow

/**
 * ## English:
 * This application is a minimal example of a merge print program that uses the layout files.
 * ### Required libraries
 * - CSJPOS_CLFX_PrintEngine.jar (Layout SDK)
 * - CSJPOSLib_Android.jar (POS Print SDK)
 * ### Note
 * - Available layout files are xml format files output using the export function of Layout Editor.
 * - Available printers depend on CSJPOSLib_Android.jar. Please use the latest version of CSJPOSLib_Android.jar.
 * - Printing, etc. of this application is processed by synchronous call. Each process blocks and waits for the thread to complete. Keep this in mind when porting.
 *
 * ## Japanese:
 * 本アプリケーションは、レイアウト ファイルを利用した差込印刷プログラムの最小限の例です。
 * ### 必要ライブラリ
 * - CSJPOS_CLFX_PrintEngine.jar(Layout SDK)
 * - CSJPOSLib_Android.jar(POS Print SDK)
 * ### 注意
 * - 利用可能なレイアウト ファイルは Layout Editor のエクスポート機能を利用して出力した xml 形式のファイルです。
 * - 利用可能なプリンターは CSJPOSLib_Android.jar に依存します。CSJPOSLib_Android.jar は最新のものをご利用ください。
 * - 本アプリケーションの印刷等は、同期型呼び出しで処理します。各処理は処理が完了するまでスレッドをブロックして待機します。移植の際は、この点に留意してください。
 *
 * ## (C) 2020 CITIZEN SYSTEMS JAPAN CO., LTD. All Rights Reserved.
 */
class MainActivity : AppCompatActivity() {

    private val selectedConnectType: Int
        get() {
            var connectType = ESCPOSConst.CMP_PORT_WiFi
            val spinnerInterface = findViewById<Spinner>(R.id.spinner_Interface)
            if (spinnerInterface != null) {
                connectType = spinnerInterface.selectedItemPosition
            }
            return connectType
        }

    private var address: String
        get() {
            var address = ""
            val editTextAddress = findViewById<EditText>(R.id.editText_Address)
            if (editTextAddress != null) {
                address = editTextAddress.text.toString()
            }
            return address
        }
        set(address) {
            val editTextAddress = findViewById<EditText>(R.id.editText_Address)
            editTextAddress?.setText(address)
        }

    private fun getSearchTime(connectType: Int): Int {
        var searchTime = 0
        when (connectType) {
            ESCPOSConst.CMP_PORT_WiFi -> searchTime = 3
            ESCPOSConst.CMP_PORT_Bluetooth, ESCPOSConst.CMP_PORT_Bluetooth_Insecure -> searchTime = 10
            ESCPOSConst.CMP_PORT_USB -> searchTime = 0
        }
        return searchTime
    }

    private fun setAddressHint(connectType: Int) {
        val editTextAddress = findViewById<EditText>(R.id.editText_Address)
        if (editTextAddress != null) {
            var hint = ""
            when (connectType) {
                ESCPOSConst.CMP_PORT_WiFi -> hint = getString(R.string.hint_address_wifi)
                ESCPOSConst.CMP_PORT_Bluetooth -> hint = getString(R.string.hint_address_bluetooth_secure)
                ESCPOSConst.CMP_PORT_Bluetooth_Insecure -> hint = getString(R.string.hint_address_bluetooth_insecure)
                ESCPOSConst.CMP_PORT_USB -> hint = getString(R.string.hint_usb_device)
            }
            editTextAddress.hint = hint
        }
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        // Software keyboard : Hidden
        window.setSoftInputMode(android.view.WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN)

        //
        getPermissions(permissions)

        //
        (findViewById<View>(R.id.editText_Header) as EditText).setText(DEFAULT_HEADER)
        (findViewById<View>(R.id.editText_Item01) as EditText).setText(DEFAULT_ITEM01)
        (findViewById<View>(R.id.editText_Item02) as EditText).setText(DEFAULT_ITEM02)
        (findViewById<View>(R.id.editText_Item03) as EditText).setText(DEFAULT_ITEM03)
        (findViewById<View>(R.id.editText_Item04) as EditText).setText(DEFAULT_ITEM04)
        (findViewById<View>(R.id.editText_Item05) as EditText).setText(DEFAULT_ITEM05)
        (findViewById<View>(R.id.editText_Footer) as EditText).setText(DEFAULT_FOOTER)

        // Print Button
        val buttonPrint = findViewById<Button>(R.id.buttonPrint)
        buttonPrint.setOnClickListener { onPrint() }

        // Select IF
        val spinnerInterface = findViewById<Spinner>(R.id.spinner_Interface)
        if (spinnerInterface != null) {
            spinnerInterface.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
                override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                    address = loadAddress(position)
                    setAddressHint(position)
                }

                override fun onNothingSelected(parent: AdapterView<*>) {
                    Log.e("OnNothingSelected", "NothingSelected")
                }
            }
            //
            spinnerInterface.setSelection(0)
        }

        // Search
        val buttonSearch = findViewById<Button>(R.id.button_Search)
        buttonSearch?.setOnClickListener { onSearch(0) }
    }


    /**
     * Search Printer Button
     * @param searchMode 0 : searchCitizenPrinter(), != 0 : searchESCPOSPrinter()
     */
    private fun onSearch(searchMode: Int) {

        // Get selected interface
        val connectType = selectedConnectType
        if (connectType == ESCPOSConst.CMP_PORT_USB) {
            // Usb interface is Auto Detect only
            toast("USB interface is Automatic Detection only", Toast.LENGTH_LONG)
            return
        }

        // Create an instance
        val printer = ESCPOSPrinter()

        // Set context
        printer.setContext(this@MainActivity)

        // Get timeout
        val timeout = getSearchTime(connectType)

        val errCode = IntArray(1)
        var items: Array<String?>? = null
        var itemsDetail: Array<String?>? = null
        // Search
        if (searchMode == 0) {
            // Search Citizen Printer
            val list1 = printer.searchCitizenPrinter(connectType, timeout, errCode)
            if (list1.isNotEmpty()) {
                items = arrayOfNulls(list1.size)
                itemsDetail = arrayOfNulls(list1.size)
                for (i in list1.indices) {
                    if (connectType == ESCPOSConst.CMP_PORT_WiFi) {
                        items[i] = list1[i].ipAddress
                        itemsDetail[i] = list1[i].ipAddress + " [" + list1[i].macAddress + "]"
                    } else {
                        items[i] = list1[i].bdAddress
                        itemsDetail[i] = list1[i].bdAddress + " [" + list1[i].deviceName + "]"
                    }
                }
            }
        } else {
            // Search Printer
            val list2 = printer.searchESCPOSPrinter(connectType, timeout, errCode)
            if (list2.isNotEmpty()) {
                items = list2
                itemsDetail = list2
            }
        }

        if (items != null) {
            var checkedItem = -1
            for (i in items.indices) {
                if (items[i] == address) {
                    checkedItem = i
                    break
                }
            }
            // Search address preview and select
            val finalItems = items
            AlertDialog.Builder(this@MainActivity)
                    .setTitle("Select the address")
                    .setSingleChoiceItems(itemsDetail, checkedItem) { dialog, which ->
                        address = String.format("%s", finalItems[which])
                        dialog.dismiss()
                    }
                    .show()
        } else {
            // 0 search results
            toast("0 Search Results", Toast.LENGTH_LONG)
        }
    }

    /**
     * Print Button
     */
    private fun onPrint() {

        // Get the port type.
        val connectType = selectedConnectType

        // Get the address.
        val address = address

        // Get the layout file.
        val spinnerLayout = findViewById<Spinner>(R.id.spinner_Layout)
        val layoutName = spinnerLayout.selectedItem as String
        val layoutId = resources.getIdentifier(layoutName, "raw", packageName)

        // Get the setting of upside down.
        val checkUpsideDown = findViewById<CheckBox>(R.id.checkBox_UpsideDown)
        val isUpsideDown = checkUpsideDown.isChecked

        // Get the setting of Logo.
        val checkReplaceLogo = findViewById<CheckBox>(R.id.checkBox_ReplaceLogo)
        val isReplaceLogo = checkReplaceLogo.isChecked

        // Gets the print data.
        val header = (findViewById<View>(R.id.editText_Header) as EditText).text.toString()
        val footer = (findViewById<View>(R.id.editText_Footer) as EditText).text.toString()
        val items = ArrayList<String>()
        items.add((findViewById<View>(R.id.editText_Item01) as EditText).text.toString())
        items.add((findViewById<View>(R.id.editText_Item02) as EditText).text.toString())
        items.add((findViewById<View>(R.id.editText_Item03) as EditText).text.toString())
        items.add((findViewById<View>(R.id.editText_Item04) as EditText).text.toString())
        items.add((findViewById<View>(R.id.editText_Item05) as EditText).text.toString())

        doPrint(connectType, address, layoutId, isUpsideDown, isReplaceLogo, header, footer, items.toTypedArray())

    }


    /**
     * Make the printing by current UI settings.
     *
     * @param connectType   : Port type of the printer. ESCPOSConst.CMP_PORT_WiFi, CMP_PORT_Bluetooth, CMP_PORT_Bluetooth_Insecure, CMP_PORT_USB
     * @param address       : Address of the printer port.
     * @param layoutId      : Resource ID of the layout file.
     * @param isUpsideDown  : Upside down printing.
     * @param isReplaceLogo : Replace Logo.
     * @param header        : Header text.
     * @param footer        : Footer text.
     * @param items         : Items name.
     */
    private fun doPrint(connectType: Int, address: String, layoutId: Int, isUpsideDown: Boolean, isReplaceLogo: Boolean, header: String, footer: String, items: Array<String>) {

        val inputStream = resources.openRawResource(layoutId)

        //
        // LayoutPrintEngine Class.
        //
        val lpe = LayoutPrintEngine()
        val result = lpe.open(inputStream)
        if (result == 0) {
            lpe.beginPrint()

            //
            // Header frame
            //
            var frameIndex = lpe.initFrame("Frame1")
            lpe.setPartsData(frameIndex, "Text1", header)
            if (isReplaceLogo) {
                val opt = android.graphics.BitmapFactory.Options()
                opt.inScaled = false
                val bmp = android.graphics.BitmapFactory.decodeResource(resources, R.raw.newlogo, opt)
                val outputStream = java.io.ByteArrayOutputStream()
                bmp.compress(android.graphics.Bitmap.CompressFormat.PNG, 100, outputStream)
                val bytes = outputStream.toByteArray()
                lpe.setPartsData(frameIndex, "Image1", bytes)
            }
            lpe.addFrame(frameIndex)

            //
            // Items frame
            //
            var digits = 1
            for (item in items) {
                frameIndex = lpe.initFrame("Frame2")
                // Set a item name.
                lpe.setPartsData(frameIndex, "Text1", item)
                // Create and set a dummy price. ( ex. 9.99, 99.99, 999.99,,, )
                val price = 10.00.pow(digits++.toDouble()) - 0.01
                val sPrice = DecimalFormat("#,##0.##").format(price)
                lpe.setPartsData(frameIndex, "Text2", sPrice)
                lpe.addFrame(frameIndex)
            }

            //
            // Footer frame
            //
            frameIndex = lpe.initFrame("Frame3")
            lpe.setPartsData(frameIndex, "Text1", footer)
            lpe.setPartsData(frameIndex, "Barcode1", footer)
            lpe.addFrame(frameIndex)

            //
            // Coupon frame, Please do not execute addFrame method, using initFrame method only.
            //
            lpe.initFrame("Frame4")

            //
            // Output to the printer
            //
            doPrint(connectType, address, lpe, isUpsideDown)

            lpe.endPrint()
            lpe.close()

        } else {
            // Error
            toast("LayoutPrintEngine open() error result : $result", Toast.LENGTH_LONG)
        }

        try {
            inputStream.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }

    }

    /**
     * Connects to the printer and outputs the print data.
     *
     * @param connectType  : Port type of the printer. ESCPOSConst.CMP_PORT_WiFi, CMP_PORT_Bluetooth, CMP_PORT_Bluetooth_Insecure, CMP_PORT_USB
     * @param address      : Address of the printer port.
     * @param lpe          : LayoutPrintEngine object that holds the print layout
     * @param isUpsideDown : Upside down.
     * @return result.
     */
    private fun doPrint(connectType: Int, address: String, lpe: LayoutPrintEngine, isUpsideDown: Boolean): Int {

        var result: Int

        //
        // ESCPOSPrinter Class.
        //
        val printer = ESCPOSPrinter()

        // Required to use the USB interface on the printer.
        printer.setContext(applicationContext)

        // Character set
//		printer.setEncoding("ISO-8859-1");	// Latin-1
        printer.setEncoding("Shift_JIS")    // Japanese
//		printer.setEncoding("UTF-8");		// Requires a printer that supports UTF-8.

        //
        // Connects to the printer.
        //
        result = printer.connect(connectType, address)
        if (result == ESCPOSConst.CMP_SUCCESS) {

            saveAddress(connectType, address)

            //
            // Outputs the print data.
            //
            result = lpe.doPrint(printer, isUpsideDown)

            //
            // ESCPOSPrinter error handling
            //
            if (result == ESCPOSConst.CMP_E_OFFLINE) {
                printer.clearOutput()
            }

            //
            // Disconnects the printer.
            //
            printer.disconnect()

            if (result == 0) {
                toast("LayoutPrintEngine doPrint() success ", Toast.LENGTH_LONG)
            } else {
                // Error
                toast("LayoutPrintEngine doPrint() error result : $result", Toast.LENGTH_LONG)
            }
        } else {
            // Error
            toast("ESCPOSPrinter connect() error result : $result", Toast.LENGTH_LONG)
        }
        return result
    }


    //
    // Save & Load Address
    //
    private fun saveAddress(connectType: Int, address: String) {
        when (connectType) {
            ESCPOSConst.CMP_PORT_WiFi -> savePreferences(PRE_KEY_IP_ADDRESS, address)
            ESCPOSConst.CMP_PORT_Bluetooth, ESCPOSConst.CMP_PORT_Bluetooth_Insecure -> savePreferences(PRE_KEY_BLUETOOTH_ADDRESS, address)
            else -> {
            }
        }
    }

    private fun loadAddress(connectType: Int): String {
        var address = ""
        when (connectType) {
            ESCPOSConst.CMP_PORT_WiFi -> address = loadPreferences(PRE_KEY_IP_ADDRESS, DEFAULT_IP_ADDRESS)
            ESCPOSConst.CMP_PORT_Bluetooth, ESCPOSConst.CMP_PORT_Bluetooth_Insecure -> address = loadPreferences(PRE_KEY_BLUETOOTH_ADDRESS, DEFAULT_BLUETOOTH_ADDRESS)
            else -> {
            }
        }
        return address
    }

    private fun savePreferences(sKeyName: String, sValue: String) {
        val sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE)
        val e = sp.edit()
        e.putString(sKeyName, sValue)
        e.apply()
    }

    private fun loadPreferences(sKeyName: String, sDefaultValue: String): String {
        val sp = getSharedPreferences(PREFERENCES_NAME, Activity.MODE_PRIVATE)
        var value = sp.getString(sKeyName, sDefaultValue)
        if (value == null) {
            value = ""
        }
        return value
    }

    private fun toast(message: String, duration: Int) {
        val toast = Toast.makeText(applicationContext, message, duration)
        toast.setGravity(Gravity.CENTER, 0, 0)
        toast.show()
    }

    /**
     * Get the permissions.
     *
     * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private fun getPermissions(permissions: Array<String>) {

        if (!checkPermissions(permissions)) {
            requestPermissions(permissions)
        } else {
            Log.d("getPermissions", "PERMISSION_GRANTED")
        }
    }

    /**
     * Checks whether this app has all permissions.
     *
     * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     * @return true:All permissions granted, false:Permission denied
     */
    private fun checkPermissions(permissions: Array<String>): Boolean {

        for (permission in permissions) {
            val permissionResult = ContextCompat.checkSelfPermission(this, permission)
            if (permissionResult != android.content.pm.PackageManager.PERMISSION_GRANTED) {
                return false                // non-permission
            }
        }
        return true
    }

    /**
     * Requests permissions to be granted to this app.
     *
     * @param permissions ex. {android.Manifest.permission.ACCESS_FINE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private fun requestPermissions(permissions: Array<String>) {
        ActivityCompat.requestPermissions(this, permissions, PERMISSION_REQUEST_CODE)
    }



    companion object {
        /**
         * Permissions
         * @see [Dangerous permissions and permission groups.](https://developer.android.com/guide/topics/security/permissions.perm-groups)
         */
        private val permissions = arrayOf(
                android.Manifest.permission.ACCESS_FINE_LOCATION
//              android.Manifest.permission.WRITE_EXTERNAL_STORAGE
        )
        /** PERMISSION_REQUEST_CODE for [ActivityCompat.OnRequestPermissionsResultCallback] @Override#onRequestPermissionsResult */
        const val PERMISSION_REQUEST_CODE = 1234

        const val PREFERENCES_NAME = "CitizenLayoutSDKSample"
        const val PRE_KEY_IP_ADDRESS = "IP_Address"
        const val DEFAULT_IP_ADDRESS = "192.168.0.100"
        const val PRE_KEY_BLUETOOTH_ADDRESS = "Bluetooth_Address"
        const val DEFAULT_BLUETOOTH_ADDRESS = "00:01:90:E8:03:33"

        const val DEFAULT_HEADER = "CITIZEN"
        const val DEFAULT_ITEM01 = "Sample ABC"
        const val DEFAULT_ITEM02 = "Sample 123"
        const val DEFAULT_ITEM03 = "Sample 456"
        const val DEFAULT_ITEM04 = "Sample 789"
        const val DEFAULT_ITEM05 = "Coupon"
        const val DEFAULT_FOOTER = "https://www.citizen-systems.co.jp"
    }
}
